@@ -1,4 +1,5 @@
 import re
+import warnings
 import functools
 
 
@@ -28,6 +29,21 @@ def valid_igt(dataset, table, column, row):
         raise ValueError('number of words and word glosses does not match')
 
 
+def valid_grammaticalityJudgement(dataset, table, column, row):
+    lid_name = dataset.readonly_column_names.ExampleTable.languageReference[0]
+    gc_name = dataset.readonly_column_names.LanguageTable.glottocode[0]
+    if row[column.name] is not None:
+        lg = dataset.get_row('LanguageTable', row[lid_name])
+        if lg[gc_name]:
+            raise ValueError('Glottolog language linked from ungrammatical example')
+
+
+def valid_mediaType(dataset, table, column, row):
+    main, _, sub = row[column.name].partition('/')
+    if not re.fullmatch('[a-z]+', main):
+        warnings.warn('Invalid main part in media type: {}'.format(main))
+
+
 VALIDATORS = [
     (
         None,
@@ -44,5 +60,13 @@ VALIDATORS = [
     (
         None,
         'http://cldf.clld.org/v1.0/terms.rdf#source',
-        valid_references)
+        valid_references),
+    (
+        None,
+        'http://cldf.clld.org/v1.0/terms.rdf#grammaticalityJudgement',
+        valid_grammaticalityJudgement),
+    (
+        None,
+        'http://cldf.clld.org/v1.0/terms.rdf#mediaType',
+        valid_mediaType),
 ]
