@@ -2,6 +2,7 @@ import shutil
 import logging
 import zipfile
 import warnings
+import mimetypes
 import contextlib
 
 import pytest
@@ -10,8 +11,8 @@ from csvw.metadata import TableGroup, ForeignKey, URITemplate, Column, Table, Li
 
 from pycldf.terms import term_uri, TERMS
 from pycldf.dataset import (
-    Generic, Wordlist, StructureDataset, Dictionary, ParallelText, Dataset, GitRepository,
-    make_column, get_modules, iter_datasets, SchemaError)
+    Generic, Wordlist, StructureDataset, Dictionary, ParallelText, Dataset, TextCorpus,
+    GitRepository, make_column, get_modules, iter_datasets, SchemaError)
 from pycldf.sources import Sources
 
 
@@ -25,6 +26,11 @@ def ds_wl(tmp_path):
     return Wordlist.in_dir(tmp_path)
 
 
+@pytest.fixture
+def ds_tc(tmp_path):
+    return TextCorpus.in_dir(tmp_path)
+
+
 @pytest.fixture
 def ds_wl_notables(tmp_path):
     return Wordlist.in_dir(str(tmp_path), empty_tables=True)
@@ -94,8 +100,9 @@ def test_provenance(ds, tmp_path):
     assert ds.properties['prov:wasDerivedFrom']['dc:created']
 
 
-def test_primary_table(ds):
+def test_primary_table(ds, ds_tc):
     assert ds.primary_table is None
+    assert ds_tc.primary_table is not None
 
 
 def test_components(ds):
@@ -171,6 +178,16 @@ def test_example_validators(ds):
         ds.validate()
 
 
+def test_invalid_mimetype(ds, recwarn):
+    ds.add_component('MediaTable')
+    ds.write(MediaTable=[{
+        'ID': '1',
+        'Media_Type': mimetypes.guess_type('f.png'),
+        'Download_URL': 'http://example.org'}])
+    ds.validate()
+    assert 'Invalid main part' in str(recwarn.pop(UserWarning).message)
+
+
 def test_regex_validator_for_listvalued_column(ds):
     ds.add_table(
         'test',
@@ -832,7 +849,7 @@ def test_get_modules():
 
 @pytest.mark.filterwarnings('ignore::UserWarning')
 def test_iter_datasets(data, tmp_path, csvw3, caplog):
-    assert len(list(iter_datasets(data))) == 9 if csvw3 else 10
+    assert len(list(iter_datasets(data))) == 11 if csvw3 else 12
 
     if csvw3:
         assert 'Reading' in caplog.records[0].msg
@@ -938,3 +955,7 @@ def test_Dataset_set_sources(ds):
     src = Sources()
     ds.sources = src
     assert ds.sources is src
+
+
+def test_StructureDataset(structuredataset_with_examples):
+    assert len(structuredataset_with_examples.features) == 2
